@@ -1,9 +1,9 @@
-# $Id: FileSuffix.pm,v 1.2 2002/02/01 14:45:07 matts Exp $
+# $Id: FileSuffix.pm,v 1.6 2001/01/14 13:18:07 matt Exp $
 
 package Apache::AxKit::StyleChooser::FileSuffix;
 
 use strict;
-use Apache::Constants qw(OK);
+use Apache::Constants;
 use Apache::URI;
 
 sub handler {
@@ -17,7 +17,7 @@ sub handler {
 #		warn "setting notes: $style\n";
 		$r->notes('preferred_style', $style);
 	}
-	return OK;
+	return DECLINED;
 }
 
 1;
@@ -27,12 +27,6 @@ __END__
 
 Apache::AxKit::StyleChooser::FileSuffix - Choose stylesheet using file suffix
 
-=head1 SYNOPSIS
-
-	PerlTypeHandler Apache::AxKit::StyleChooser::FileSuffix
-	SetHandler perl-script
-	PerlHandler AxKit
-
 =head1 DESCRIPTION
 
 This module lets you pick a stylesheet based on the filename suffix. To use
