@@ -1,9 +1,9 @@
-# $Id: PathInfo.pm,v 1.3 2002/07/30 10:33:42 jwalt Exp $
+# $Id: PathInfo.pm,v 1.5 2001/01/14 15:48:19 matt Exp $
 
 package Apache::AxKit::StyleChooser::PathInfo;
 
 use strict;
-use Apache::Constants qw(OK);
+use Apache::Constants;
 use Apache::URI;
 
 sub handler {
@@ -29,7 +29,7 @@ sub handler {
 #		warn "setting notes: $style\n";
 		$r->notes('preferred_style', $style);
 	}
-	return OK;
+	return DECLINED;
 }
 
 1;
@@ -39,31 +39,20 @@ __END__
 
 Apache::AxKit::StyleChooser::PathInfo - Choose stylesheet using PATH_INFO
 
-=head1 SYNOPSIS
-
-  AxAddPlugin Apache::AxKit::StyleChooser::PathInfo
-
 =head1 DESCRIPTION
 
-This module lets you pick a stylesheet based on the extra PATH_INFO. 
-To use it, simply add this module as an AxKit plugin that 
-will be run before main AxKit processing is done.
+This module lets you pick a stylesheet based on the extra PATH_INFO. To use
+it, simply add this module to the list of PerlHandlers prior to
+AxKit:
 
-  AxAddPlugin Apache::AxKit::StyleChooser::PathInfo  
+	PerlHandler Apache::AxKit::StyleChooser::PathInfo \
+			AxKit
 
 Then simply by referencing your xml files as follows:
 
-   http://xml.server.com/myfile.xml/printable
+	http://xml.server.com/myfile.xml/printable
 
 You will recieve the alternate stylesheets with title "printable". See 
 the HTML 4.0 specification for more details on stylesheet choice.
 
-See the B<AxStyleName> AxKit configuration directive
-for more information on how to setup named styles.
-
-=head1 SEE ALSO
-
-AxKit.
-
 =cut
-
