@@ -1,4 +1,4 @@
-# $Id: UserAgent.pm,v 1.2 2002/02/01 14:45:07 matts Exp $
+# $Id: UserAgent.pm,v 1.2 2001/01/09 14:50:19 kip Exp $
 
 package Apache::AxKit::StyleChooser::UserAgent;
 
@@ -7,7 +7,7 @@ use vars qw($VERSION);
 
 $VERSION = '0.01';
 
-use Apache::Constants qw(OK);
+use Apache::Constants;
 
 sub handler {
     my $r = shift;
@@ -27,7 +27,7 @@ sub handler {
         }
     }
 
-    return OK;
+    return DECLINED;
 }
 
 1;
@@ -38,7 +38,6 @@ __END__
 Apache::AxKit::StyleChooser::UserAgent - Choose stylesheets based on the user agent.
 
 =head1 SYNOPSIS
-
     In your .conf or .htaccess file(s):
     
     PerlHandler Apache::AxKit::StyleChooser::UserAgent \
